# Import necessary libraries
import numpy as np
import seaborn as sns
import pandas as pd
import openml
import os
import requests
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import accuracy_score
from sklearn.datasets import fetch_openml

from sklearn.datasets import load_svmlight_file

from sklearn.preprocessing import OneHotEncoder

from sklearn.datasets import make_blobs, make_classification


def generate_dataset(problem, n_samples=1000, n_agents=10):

    rng=np.random.default_rng(seed=42)
    
    if problem == "noisy":
        X, y = make_blobs(n_samples=n_samples, centers=([-2]*20,[0,4]*10),
                          n_features=20, cluster_std=10,
                          random_state=42)

        y[y>0]=1

        data_homogeneous = [(X,y) for _ in range(n_agents)]

        indices = np.array_split(np.arange(len(y)), n_agents)
        data_heterogeneous = [(X[idx], y[idx]) for idx in indices]



    elif problem == "heterogeneous":
        data_heterogeneous = []
        
        X1, y1 = make_blobs(n_samples=n_samples//2, centers=2,
                            n_features=20, cluster_std=1,
                            random_state=42)
        X2 = X1 + 0.1 * rng.normal(size=X1.shape)
        X22 = X2.copy()

        rng.shuffle(X2)
        y2 = 1 - y1

        X = np.vstack((X1, X22))
        y = np.hstack((y1, y1))


        idx_minus = np.arange(len(y1))
        idx_plus = np.arange(len(y2))

        data_heterogeneous  = [(X1[idx], y1[idx]) for idx in np.array_split(idx_minus, n_agents//2 + n_agents % 2)]
        data_heterogeneous += [(X2[idx], y2[idx]) for idx in np.array_split(idx_plus, n_agents//2)]
                                              
        data_homogeneous = [(X, y) for _ in range(n_agents)]


    else:
        print("Please enter a valid dataset.")
        exit(0)
        
    return {"homogeneous": data_homogeneous,
            "heterogeneous": data_heterogeneous}

            
