import numpy as np
import os
import pickle
import sys

from algos import *
from datasets import *

from multiprocessing.pool import Pool

# general parameters 
n_local_list = [1, 10, 20, 50, 100]
n_agents = 10
n_runs = 10

# read args
if len(sys.argv) != 2:
    print("Usage: python experiment.py <dataset>")
    sys.exit(1)
problem = sys.argv[1]


# Simulation parameters
alpha = 0.01
n_rounds = 10000
batch_size = 1


# rng for reproducibility
data_rng = np.random.default_rng(seed=42)

# generate data
datas = generate_dataset(problem)

# number of records and features
n_records, n_features = datas["homogeneous"][0][0].shape



# problem setting and optimal parameters
lambda_ = 1e-4
theta_star = {"homogeneous": compute_theta_star(datas["homogeneous"], lambda_),
              "heterogeneous": compute_theta_star(datas["heterogeneous"], lambda_)}
theta = np.zeros(n_features) 
    
kwargs = {
    "theta": theta+1, #initialize not too close to optimum
    "lambda_": lambda_,
    "batch_size": batch_size
}


def run_algo(init_seed):
    results_dir = "plots/results/" + problem + "/" + str(n_rounds) + "_" + str(alpha) + "/"
    os.makedirs(results_dir + "homogeneous" + "/" + str(init_seed), exist_ok=True)
    os.makedirs(results_dir + "heterogeneous" + "/" + str(init_seed), exist_ok=True)

    seed_rng = np.random.default_rng(seed=init_seed)


    data_type = "heterogeneous"
    data = datas[data_type]

    
    for n_local in n_local_list:
        print("H=", n_local, "data_type=", data_type)
        seed = seed_rng.integers(1e6)
        n_rounds_adapted = n_rounds // n_local
            
        thetas_fedavg = fed_avg(clients_data=data, step=alpha,
                                seed=seed, n_local=n_local, n_rounds=n_rounds_adapted,
                                **kwargs)
            
        thetas_fedavg_2alpha = fed_avg(clients_data=data, step=2*alpha,
                                       seed=seed, n_local=n_local, n_rounds=n_rounds_adapted,
                                       **kwargs)
        
        
        thetas_scaffold = scaffold(clients_data=data, step=alpha,
                                   seed=seed, n_local=n_local, n_rounds=n_rounds_adapted,
                                   **kwargs)
            
    
        with open(results_dir + data_type + "/" + str(init_seed) + "/" + str(n_local) + '.pkl', 'wb') as f:
            pickle.dump({"n_local": n_local,
                         "theta_star": theta_star[data_type],
                         "theta_fedavg": thetas_fedavg,
                         "theta_scaffold": thetas_scaffold,
                         "theta_fedavg_rr_gamma": 2*thetas_fedavg-thetas_fedavg_2alpha
                         }, f)
            


seeds = [s for s in range(n_runs)]

with Pool(10) as p:
    print(p.map(run_algo, seeds))
